package space.rocketnine.gophast;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.RecyclerView;
import android.text.format.Formatter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

public class DownloadAdapter extends RecyclerView.Adapter<DownloadAdapter.DownloadViewHolder> {
    private Context context;
    private ArrayList<Download> downloads;
    private GoPhast gophast;
    private Activity activity;

    public static class DeltaSize {
        long size;

        public DeltaSize(long s) {
            size = s;
        }
    }


    public static class DeltaProgress {
        int progress;

        public DeltaProgress(int p) {
            progress = p;
        }
    }

    public static class DownloadViewHolder extends RecyclerView.ViewHolder {
        public View v;
        // each data item is just a string in this case
        public TextView txtName;
        public TextView txtSize;
        public TextView txtURL;
        public ProgressBar progressBar;

        public DownloadViewHolder(LinearLayout v) {
            super(v);
            this.v = v;
            txtName = v.findViewById(R.id.downloadName);
            txtSize = v.findViewById(R.id.downloadSize);
            txtURL = v.findViewById(R.id.downloadURL);

            progressBar = v.findViewById(R.id.downloadProgress);
            progressBar.setMax(10000);
        }
    }

    public DownloadAdapter(GoPhast gp, Activity a, ArrayList<Download> ds) {
        gophast = gp;
        context = a;
        activity = a;
        downloads = ds;
    }

    @Override
    public DownloadAdapter.DownloadViewHolder onCreateViewHolder(ViewGroup parent,
                                                                 int viewType) {
        // create a new view
        LinearLayout v = (LinearLayout) LayoutInflater.from(parent.getContext())
                .inflate(R.layout.list_downloads, parent, false);
        DownloadViewHolder vh = new DownloadViewHolder(v);
        return vh;
    }

    // Replace the contents of a view (invoked by the layout manager)
    @Override
    public void onBindViewHolder(final DownloadViewHolder holder, int position) {
        Download d = downloads.get(position);
        holder.txtName.setText(d.Name);
        if (d.Size > 0) {
            holder.txtSize.setText(Formatter.formatShortFileSize(context, d.Size));
            holder.txtSize.setVisibility(View.VISIBLE);
        } else {
            holder.txtSize.setVisibility(View.INVISIBLE);
        }
        holder.txtURL.setText(d.URL);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            holder.progressBar.setProgress((int) (d.Downloaded * 10000.0 / d.Size), true);
        } else {
            holder.progressBar.setProgress((int) (d.Downloaded * 10000.0 / d.Size));
        }

        holder.v.setOnClickListener(v -> {
            Download d1 = downloads.get(holder.getAdapterPosition());
            if (d1 == null) {
                return;
            }

            Intent intent = new Intent();
            intent.setAction(Intent.ACTION_VIEW);
            intent.setDataAndType(d1.Path, d1.getMIME());
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);

            try {
                context.startActivity(intent);
            } catch (Exception e) {
                e.printStackTrace();
                getApp().longToast("No application able to open this file is installed");
            }
        });
        holder.v.setOnLongClickListener(v -> {
            Download d12 = downloads.get(holder.getAdapterPosition());
            if (d12 == null) {
                return true;
            }

            AlertDialog downloadDialog = new AlertDialog.Builder(activity, R.style.AppTheme_Dialog)
                    .setTitle(d12.Name)
                    .setItems(R.array.downloadOptions, (dialogInterface, i) -> {
                        Download d121 = downloads.get(holder.getAdapterPosition());
                        if (d121 == null) {
                            return;
                        }

                        if (i == 0) {
                            Intent intent = new Intent();
                            intent.setAction(Intent.ACTION_VIEW);
                            intent.setDataAndType(d121.Path, d121.getMIME());
                            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                            context.startActivity(Intent.createChooser(intent, context.getString(R.string.openWith)));
                        } else if (i == 1) {
                            getApp().removeDownload(d121);
                        }
                    })
                    .create();
            downloadDialog.show();

            return true;
        });
    }

    @Override
    public void onBindViewHolder(final DownloadViewHolder holder, int position, List<Object> payloads) {
        if (payloads.isEmpty()) {
            onBindViewHolder(holder, position);
        } else {
            for (Object payload : payloads) {
                if (payload instanceof DeltaProgress) {
                    DeltaProgress d = (DeltaProgress) payload;

                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                        holder.progressBar.setProgress(d.progress, true);
                    } else {
                        holder.progressBar.setProgress(d.progress);
                    }
                } else if (payload instanceof DeltaSize) {
                    DeltaSize d = (DeltaSize) payload;

                    if (d.size > 0) {
                        holder.txtSize.setText(Formatter.formatShortFileSize(context, d.size));
                        gophast.animateView(context, holder.txtSize, R.anim.fade_in);
                    } else {
                        holder.txtSize.setVisibility(View.INVISIBLE);
                    }
                }
            }
        }
    }

    // Return the size of your dataset (invoked by the layout manager)
    @Override
    public int getItemCount() {
        return downloads.size();
    }

    private GoPhast getApp() {
        return gophast;
    }
}