package space.rocketnine.gophast;

import android.net.Uri;
import android.os.Build;
import android.os.Process;
import android.support.v7.widget.RecyclerView;
import android.util.Log;

import manager.Manager;
import manager.NewDownloadInfo;

class DownloadRunnable implements Runnable {
    Download d;
    GoPhast gophast;
    RecyclerView recyclerView;

    private Runnable notifySizeRunnable = new Runnable() {
        @Override
        public void run() {
            gophast.adapter.notifyItemChanged(d.Index, new DownloadAdapter.DeltaSize(d.Size));
        }
    };
    private Runnable notifyProgressRunnable = new Runnable() {
        @Override
        public void run() {
            gophast.adapter.notifyItemChanged(d.Index, new DownloadAdapter.DeltaProgress((int) (d.Downloaded * 10000.0 / d.Size)));
        }
    };

    @Override
    public void run() {
        Process.setThreadPriority(Process.THREAD_PRIORITY_BACKGROUND);
        try {
            Log.d("gophast", "Initiating download " + d.URL);
            NewDownloadInfo info = new NewDownloadInfo();
            info.setURL(d.URL);
            info.setPostData(new byte[]{});

            d.ID = Manager.addDownload(info);

            gophast.acquireWakeLock(d.ID);

            d.Name = Manager.getName(d.ID);
            d.Size = Manager.getSize(d.ID);

            Log.d("gophast", "Started download " + d.Name);

            if (d.Size < 0) {
                Log.d("gophast", "size < 0");
                gophast.releaseWakeLock(d.ID);
                return;
            }

            recyclerView.post(notifySizeRunnable);

            long size = d.Size;
            long id = d.ID;
            long lastDownloaded = -99;
            long newDownloaded;
            boolean active;
            for (; ; ) {
                newDownloaded = Manager.getDownloaded(id);
                active = lastDownloaded != newDownloaded;
                if (active) {
                    d.Downloaded = newDownloaded;
                    lastDownloaded = newDownloaded;
                    recyclerView.post(notifyProgressRunnable);

                    if (newDownloaded < 0 || newDownloaded == size) {
                        d.Complete = true; // TODO: Cleanup when complete
                        break;
                    }
                }

                try {
                    Thread.sleep(active ? 25 : 75);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                    break;
                }
            }

            d.Status = Manager.waitDownload(d.ID);
            if (d.Status > 0) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    d.DownloadManagerID = getApp().DownloadManager.addCompletedDownload(d.Name, d.URL, true, d.getMIME(), d.Path.toString(), d.Size, true, Uri.parse(d.URL), null);
                } else {
                    d.DownloadManagerID = getApp().DownloadManager.addCompletedDownload(d.Name, d.URL, true, d.getMIME(), d.Path.toString(), d.Size, true);
                }

                d.Downloaded = d.Size;
            } else {
                Log.d("gophast", "Status <= 0 " + d.Status);
                d.Downloaded = 0;
            }
        } catch (Exception e) {
            Log.d("gophast", "Failed to download " + d.URL);
            getApp().longToast(e.getMessage());
            e.printStackTrace();
        }

        if (d != null && d.ID > 0) {
            gophast.releaseWakeLock(d.ID);
        }
    }

    private GoPhast getApp() {
        return gophast;
    }
}
