package space.rocketnine.gophast;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceFragment;
import android.support.annotation.LayoutRes;
import android.support.annotation.Nullable;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.Toolbar;
import android.text.format.Formatter;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import com.nononsenseapps.filepicker.FilePickerActivity;
import com.nononsenseapps.filepicker.Utils;

import java.io.File;
import java.util.List;

public class SettingsActivity extends PreferenceActivity {
    private static final boolean FORCE_TABLET_VIEW = false;
    public DownloadPreferenceFragment downloadPreferenceFragment = null;
    private AppCompatDelegate mDelegate;
    private GoPhast gophast;
    private final int RequestDownloadDir = 1995;
    private Preference prefDownloadDir;

    private Preference.OnPreferenceChangeListener onPreferenceChangeListener = new Preference.OnPreferenceChangeListener() {
        @Override
        public boolean onPreferenceChange(Preference preference, Object value) {
            if (preference.getKey().equals("pref_downloaddir")) {
                String downloadDir;
                if (value != null && !value.toString().isEmpty()) {
                    downloadDir = value.toString();
                } else {
                    downloadDir = getApp().getDownloadDir();
                }

                getApp().setDownloadDir(downloadDir);
                preference.setSummary(downloadDir);
            } else if (preference.getKey().equals("pref_minsplitsize")) {
                preference.setSummary(Formatter.formatShortFileSize(SettingsActivity.this, Long.valueOf(value.toString())));
                if (Long.valueOf(value.toString()) >= 0) {
                    manager.Manager.setMinSplitSize(Long.valueOf(value.toString()));
                }
            } else if (preference.getKey().equals("pref_maxconnections")) {
                preference.setSummary(String.format(getString(R.string.maxconnections_summary), value.toString()));
                if (Long.valueOf(value.toString()) >= 0) {
                    manager.Manager.setMaxConnections(Long.valueOf(value.toString()));
                }
            }

            return true;
        }
    };

    private void bindPreferenceSummaryToValue(Preference preference) {
        // Set the listener to watch for value changes.
        preference.setOnPreferenceChangeListener(onPreferenceChangeListener);

        if (preference.getKey().equals("pref_downloaddir")) {
            onPreferenceChangeListener.onPreferenceChange(preference, getApp().getPrefs().getString(preference.getKey(), ""));
        } else if (preference.getKey().equals("pref_minsplitsize")) {
            onPreferenceChangeListener.onPreferenceChange(preference, getApp().getPrefs().getString(preference.getKey(), "10485760"));
        } else if (preference.getKey().equals("pref_maxconnections")) {
            onPreferenceChangeListener.onPreferenceChange(preference, getApp().getPrefs().getString(preference.getKey(), "3"));
        }
    }

    private AppCompatDelegate getDelegate() {
        if (mDelegate == null) {
            mDelegate = AppCompatDelegate.create(this, null);
        }
        return mDelegate;
    }

    public ActionBar getSupportActionBar() {
        return getDelegate().getSupportActionBar();
    }

    private static boolean isXLargeTablet(Context context) {
        return FORCE_TABLET_VIEW || ((context.getResources().getConfiguration().screenLayout
                & Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_XLARGE);
    }

    @SuppressWarnings("BooleanMethodIsAlwaysInverted")
    private static boolean isSimplePreferences(Context context) {
        return !isXLargeTablet(context);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        getDelegate().installViewFactory();
        getDelegate().onCreate(savedInstanceState);
        getSupportActionBar().setBackgroundDrawable(getResources().getDrawable(R.drawable.road));
        super.onCreate(savedInstanceState);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_settings, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent intent;
        int i = item.getItemId();
        if (i == R.id.menuAbout) {
            intent = new Intent(SettingsActivity.this, AboutActivity.class);
            startActivity(intent);
            overridePendingTransition(R.anim.slide_in, R.anim.slide_out);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        getDelegate().onPostCreate(savedInstanceState);

        setupSimplePreferencesScreen();
    }

    public void setSupportActionBar(@Nullable Toolbar toolbar) {
        getDelegate().setSupportActionBar(toolbar);
    }

    @Override
    public MenuInflater getMenuInflater() {
        return getDelegate().getMenuInflater();
    }

    @Override
    public void setContentView(@LayoutRes int layoutResID) {
        getDelegate().setContentView(layoutResID);
    }

    @Override
    public void setContentView(View view) {
        getDelegate().setContentView(view);
    }

    @Override
    public void setContentView(View view, ViewGroup.LayoutParams params) {
        getDelegate().setContentView(view, params);
    }

    @Override
    public void addContentView(View view, ViewGroup.LayoutParams params) {
        getDelegate().addContentView(view, params);
    }

    @Override
    protected void onPostResume() {
        super.onPostResume();
        getDelegate().onPostResume();
    }

    @Override
    protected void onTitleChanged(CharSequence title, int color) {
        super.onTitleChanged(title, color);
        getDelegate().setTitle(title);
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        getDelegate().onConfigurationChanged(newConfig);
    }


    @Override
    public void onStart() {
        super.onStart();
    }

    @Override
    protected void onStop() {
        super.onStop();
        getDelegate().onStop();
    }

    @Override
    public void finish() {
        super.finish();
        overridePendingTransition(R.anim.slide_in_rev, R.anim.slide_out_rev);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        getDelegate().onDestroy();
    }

    public void invalidateOptionsMenu() {
        getDelegate().invalidateOptionsMenu();
    }

    private void setupSimplePreferencesScreen() {
        if (!isSimplePreferences(this)) {
            return;
        }

        // In the simplified UI, fragments are not used at all and we instead
        // use the older PreferenceActivity APIs.

        // Add 'Download' preferences.
        PreferenceCategory fakeHeader = new PreferenceCategory(this);
        addPreferencesFromResource(R.xml.pref_download);

        setupDownloadPreferences(null);
        setupPreferences("all", null);
    }

    @Override
    public boolean onIsMultiPane() {
        return isXLargeTablet(this) && !isSimplePreferences(this);
    }

    @Override
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public void onBuildHeaders(List<Header> target) {
        if (!isSimplePreferences(this)) {
            loadHeadersFromResource(R.xml.pref_headers, target);
        }
    }


    @Override
    protected boolean isValidFragment(String fragmentName) {
        return DownloadPreferenceFragment.class.getName().equals(fragmentName);
    }

    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class DownloadPreferenceFragment extends PreferenceFragment {
        public DownloadPreferenceFragment() {
        }

        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.pref_download);

            SettingsActivity settingsactivity = (SettingsActivity) getActivity();
            settingsactivity.setupDownloadPreferences(this);
            settingsactivity.setupPreferences("download", this);
        }
    }

    void setupDownloadPreferences(PreferenceFragment preferenceFragment) {

    }

    void setupPreferences(String pref_type, PreferenceFragment preferenceFragment) {
        if (pref_type.equals("all") || pref_type.equals("download")) {
            if (preferenceFragment != null) {
                downloadPreferenceFragment = (DownloadPreferenceFragment) preferenceFragment;
            }

            (preferenceFragment == null ? findPreference("pref_downloaddir") : preferenceFragment.findPreference("pref_downloaddir")).setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    Intent i = new Intent(SettingsActivity.this, FilePickerActivity.class);
                    // This works if you defined the intent filter
                    // Intent i = new Intent(Intent.ACTION_GET_CONTENT);

                    // Set these depending on your use case. These are the defaults.
                    i.putExtra(FilePickerActivity.EXTRA_ALLOW_MULTIPLE, false);
                    i.putExtra(FilePickerActivity.EXTRA_ALLOW_CREATE_DIR, true);
                    i.putExtra(FilePickerActivity.EXTRA_MODE, FilePickerActivity.MODE_DIR);

                    String startPath = getApp().getDownloadDir();
                    boolean validStartPath = false;
                    try {
                        File f = new File(startPath);
                        validStartPath = f.isDirectory();
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    if (!validStartPath) {
                        startPath = getApp().getDefaultDownloadDir();
                    }
                    i.putExtra(FilePickerActivity.EXTRA_START_PATH, startPath);

                    startActivityForResult(i, RequestDownloadDir);
                    return false;
                }
            });

            prefDownloadDir = preferenceFragment == null ? findPreference("pref_downloaddir") : preferenceFragment.findPreference("pref_downloaddir");

            bindPreferenceSummaryToValue(preferenceFragment == null ? findPreference("pref_downloaddir") : preferenceFragment.findPreference("pref_downloaddir"));
            bindPreferenceSummaryToValue(preferenceFragment == null ? findPreference("pref_minsplitsize") : preferenceFragment.findPreference("pref_minsplitsize"));
            bindPreferenceSummaryToValue(preferenceFragment == null ? findPreference("pref_maxconnections") : preferenceFragment.findPreference("pref_maxconnections"));
        }
    }

    public void onActivityResult(int requestCode, int resultCode, Intent intent) {
        if (requestCode == RequestDownloadDir && resultCode == Activity.RESULT_OK) {
            // Use the provided utility method to parse the result
            List<Uri> files = Utils.getSelectedFilesFromResult(intent);
            for (Uri uri : files) {
                File file = Utils.getFileForUri(uri);

                getApp().getPrefs().edit().putString("pref_downloaddir", file.getAbsolutePath()).apply();
                onPreferenceChangeListener.onPreferenceChange(prefDownloadDir, file.getAbsolutePath());
            }
        }
    }

    private GoPhast getApp() {
        if (gophast == null) {
            gophast = (GoPhast) getApplication();
        }

        return gophast;
    }
}
