package space.rocketnine.gophast;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.PowerManager;
import android.util.Log;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

class WakeLocker {
    private List<Long> wakeLocks = new CopyOnWriteArrayList<Long>();

    private PowerManager.WakeLock wakeLockScreenOff;
    private PowerManager.WakeLock wakeLockScreenOn;

    @SuppressLint({"WakelockTimeout"})
    void acquire(Context ctx, long id, Boolean turnScreenOn) {
        Log.d("gophast", "WAKELOCKER: Acquiring wakelock " + id);
        wakeLocks.add(id);

        if (wakeLockScreenOff == null || wakeLockScreenOn == null) {
            PowerManager pm = (PowerManager) ctx.getSystemService(Context.POWER_SERVICE);
            if (wakeLockScreenOff == null) {
                wakeLockScreenOff = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "gophast:screenoff");
            }
            if (wakeLockScreenOn == null) {
                wakeLockScreenOn = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK | PowerManager.ON_AFTER_RELEASE, "gophast:screenon");
            }
        }

        if (!turnScreenOn) {
            if (!wakeLockScreenOff.isHeld()) {
                wakeLockScreenOff.acquire();
            }
        } else {
            if (!wakeLockScreenOn.isHeld()) {
                wakeLockScreenOn.acquire();
            }
        }
    }

    void release(Long id) {
        Log.d("gophast", "WAKELOCKER: Releasing wakelock " + id);
        wakeLocks.remove(id);

        if (wakeLocks.isEmpty()) {
            if (wakeLockScreenOff != null && wakeLockScreenOff.isHeld()) {
                wakeLockScreenOff.release();
            }
            if (wakeLockScreenOn != null && wakeLockScreenOn.isHeld()) {
                wakeLockScreenOn.release();
            }
        }
    }

    void releaseAll() {
        Log.d("gophast", "WAKELOCKER: Releasing all wakelocks");

        for (Long id : wakeLocks) {
            release(id);
        }
    }
}